package UI.GameState;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.sql.Struct;
import java.util.ArrayList;


public class WallState {
    private int frameX, frameY;

    private String source;
    private String[] lines;
    private int numRows, numColumns, w, h;
    private final int wallWidth = 5;
    private ArrayList<WallData> walls = new ArrayList<>();
    private Image wall1 ;
    private Image wall2 ;
    private Image blank ;


    public WallState(int GAME_WIDTH, int GAME_HEIGHT) {
        frameX = GAME_WIDTH;
        frameY = GAME_HEIGHT;

        try {
            source = new String(Files.readAllBytes(Paths.get("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\maps\\map3.txt")), StandardCharsets.UTF_8);
            wall1 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall1.png"));
            wall2 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall2.png"));
            blank = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\blank.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        lines = source.split("\r\n|\r|\n");

        numRows = lines.length;  //number of rows
        numColumns = lines[0].trim().length();  //number of columns

        h = (frameY-30) / (numRows);   //sideY
        w = (frameX-10) / (numColumns);  //sideX

    }


    public void paintComponent(Graphics g) {

        Image wall1scaled = wall1.getScaledInstance(w, h, java.awt.Image.SCALE_SMOOTH);
        Image wall2scaled = wall2.getScaledInstance(w, h, java.awt.Image.SCALE_SMOOTH);
        Image blankScaled = blank.getScaledInstance(w, h, java.awt.Image.SCALE_SMOOTH);
        // Draw background
        walls.clear();
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, frameX, frameY);
        g.setColor(Color.black);  //colour of the walls
        int x = 0;
        int y = 0;

        for (int r = 0; r < numRows; r++) {
            x = 0 ;
            String rowChars = lines[r];
            for (int c = 0; c < numColumns; c++) {
                if (rowChars.charAt(c) == '0') {
                    drawBlanks(g, x, y ,blankScaled);
                    x += w;
                }
                else if (rowChars.charAt(c) == '1') {
                    drawAndSaveWalls(g, x, y, r , c, wall1scaled);
                    x += w;
                }
                else if (rowChars.charAt(c) == '2') {
                    drawAndSaveWalls(g, x, y, r , c, wall2scaled);
                    x += w;
                }
            }
            y += h;
        }
    }



    private void drawAndSaveWalls(Graphics g2d,int x,int y , int r , int c ,Image image){
        WallData wall = new WallData(x , y);
        if ((c<numColumns-1 && lines[r].charAt(c+1)=='1') || (c>0 && lines[r].charAt(c-1)=='1')||(c<numColumns-1 && lines[r].charAt(c+1)=='2') || (c>0 && lines[r].charAt(c-1)=='2')){ //horizontal
            wall.setHorizontal(true);
        }
        if ((r>0 && lines[r-1].charAt(c)=='1') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='1')||(r>0 && lines[r-1].charAt(c)=='2') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='2')) { //vertical
            wall.setVertical(true);
        }
        walls.add(wall);
        g2d.drawImage(image , x , y , null);
    }

    private void drawBlanks(Graphics g2d,int x,int y,Image image){
        g2d.drawImage(image , x , y , null);
    }

    public WallData getWallForBullet(int x,int y){
        for(WallData d : walls){
            if (d.isWallForBullet(x,y))
                return d;
        }
        return null;
    }


    public boolean isWallForTank(int x,int y){
        for(WallData d : walls){
            if (d.isWallForTank(x,y))
                return true;
        }
        return false;
    }



    public ArrayList<WallData> getWalls() {
        return walls;
    }

    public class WallData {
        private int x,y;
        private boolean horizontal = false;
        private boolean vertical = false;

        public WallData(int x,int y){
            this.x = x;
            this.y = y;
        }

        public boolean isWallForTank (int x,int y){
            //BUG
            return (x>this.x-w/2 && x<this.x+w && y>this.y-w/2 && y<this.y+h);
        }

        public boolean isWallForBullet (int x,int y){
            //BUG
            return (x>this.x-5 && x<this.x+w+5 && y>this.y-5 && y<this.y+h+5);
        }

        public boolean isHorizontal() {
            return horizontal;
        }

        public void setHorizontal(boolean horizontal) {
            this.horizontal = horizontal;
        }

        public boolean isVertical() {
            return vertical;
        }

        public void setVertical(boolean vertical) {
            this.vertical = vertical;
        }
    }
}
